@@ -4,10 +4,9 @@ pub use crate::protocol::{
 };
 use crate::{
     auth::Permissions,
-    channel::Channel,
+    channel::{Channel, K5CtxWrap},
     chars::Chars,
     config::Config,
-    os::{self, Krb5Ctx, ServerCtx},
     path::Path,
     pool::{Pool, Pooled},
     protocol::{self, publisher},
@@ -16,6 +15,7 @@ use crate::{
 };
 use anyhow::{anyhow, Error, Result};
 use bytes::Buf;
+use cross_krb5::ServerCtx;
 use futures::{
     channel::{
         mpsc::{
@@ -1416,13 +1416,9 @@ async fn hello_client(
             Auth::Anonymous => bail!("authentication not supported"),
             Auth::Krb5 { upn, spn } => {
                 let p = spn.as_ref().or(upn.as_ref()).map(|s| s.as_str());
-                let ctx = os::create_server_ctx(p)?;
-                let tok = ctx
-                    .step(Some(&*tok))?
-                    .map(|b| utils::bytes(&*b))
-                    .ok_or_else(|| anyhow!("expected step to generate a token"))?;
-                con.send_one(&Token(tok)).await?;
-                con.set_ctx(ctx).await;
+                let (ctx, tok) = task::block_in_place(|| ServerCtx::accept(p, &*tok))?;
+                con.send_one(&Token(utils::bytes(&*tok))).await?;
+                con.set_ctx(K5CtxWrap::new(ctx)).await;
                 client_arrived(publisher);
             }
         },
@@ -1641,7 +1637,9 @@ async fn publish_loop(
                 }
             }
             if to_unpublish_default.len() > 0 {
-                if let Err(e) = resolver.unpublish_default(to_unpublish_default.drain()).await {
+                if let Err(e) =
+                    resolver.unpublish_default(to_unpublish_default.drain()).await
+                {
                     error!("failed to unpublish default some paths {} will retry", e)
                 }
             }
