@@ -1,10 +1,9 @@
 pub use crate::protocol::value::{FromValue, Typ, Value};
 use crate::{
     batch_channel::{self, BatchReceiver, BatchSender},
-    channel::{Channel, ReadChannel, WriteChannel},
+    channel::{Channel, K5CtxWrap, ReadChannel, WriteChannel},
     chars::Chars,
     config::Config,
-    os::{self, ClientCtx, Krb5Ctx},
     pack::{Pack, PackError},
     path::Path,
     pool::{Pool, Pooled},
@@ -19,6 +18,7 @@ use crate::{
 };
 use anyhow::{anyhow, Error, Result};
 use bytes::{Buf, BufMut, Bytes};
+use cross_krb5::ClientCtx;
 use futures::{
     channel::{
         mpsc::{self, Receiver, Sender, UnboundedReceiver, UnboundedSender},
@@ -1289,22 +1289,16 @@ async fn hello_publisher(
         }
         Auth::Krb5 { upn, .. } => {
             let p = upn.as_ref().map(|p| p.as_str());
-            let ctx = os::create_client_ctx(p, target_spn)?;
-            let tok = ctx
-                .step(None)?
-                .map(|b| utils::bytes(&*b))
-                .ok_or_else(|| anyhow!("expected step to generate a token"))?;
-            con.send_one(&Hello::Token(tok)).await?;
+            let (ctx, tok) = task::block_in_place(|| ClientCtx::initiate(p, target_spn))?;
+            con.send_one(&Hello::Token(utils::bytes(&*tok))).await?;
             match con.receive().await? {
                 Hello::Anonymous => bail!("publisher failed mutual authentication"),
                 Hello::ResolverAuthenticate(_, _) => bail!("protocol error"),
                 Hello::Token(tok) => {
-                    if ctx.step(Some(&*tok))?.is_some() {
-                        bail!("unexpected second token from step");
-                    }
+                    let ctx = task::block_in_place(|| ctx.finish(&*tok))?;
+                    con.set_ctx(K5CtxWrap::new(ctx)).await
                 }
             }
-            con.set_ctx(ctx.clone()).await;
         }
     }
     Ok(())
